<?php
// +----------------------------------------------------------------------
// | ThinkMIMI - 框架数据库模型层核心操作类
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://xiuxian.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------
header('Content-Type: text/html; charset=utf-8');

class Db {
    /**
     * PDO链接实例
     */
    private static $_PDO;
    /**
     * 表名带前缀
     */
    private static $_TABLE;
    /**
     * 数据容器
     */
    private static $_DATA = [];
    /**
     * JOIN条件
     */
    private static $_JOIN;
    /**
     * 条件节点
     */
    private static $_WHERE;
    /**
     * 查询返回的字段
     */
    private static $_FIELD;
    /**
     * 排序的字段
     */
    private static $_ORDER;
    /**
     * 分页的内容
     */
    private static $_LIMIT;
    /**
     * 最终组装后的SQL语句
     */
    private static $_SQL;
    /**
     * 预注入参数容器
    */
    private static $_PREPARE = [];
    /**
     * 是否启动预注入
    */
    private static $_STATUS;
    /**
     * 数据库配置文件
    */
    private static $_CONFIG;
    

    /**
     * 初始化PDO链接
     */
    public function __construct() {
        if (!self::$_PDO) {
            try {
                $DSN = 'mysql:dbname='.self::$_CONFIG['database'].';host='.self::$_CONFIG['hostname'].';port='.self::$_CONFIG['hostport'];
                $PDO = new \PDO($DSN, self::$_CONFIG['username'], self::$_CONFIG['password']);
                $PDO->query('SET NAMES '.self::$_CONFIG['charset'].';');
                self::$_PDO =$PDO;
            } catch(PDOException $e) {
                die ('数据库连接失败，错误信息：'.$e->getMessage());
            }
        }
    }

    /**
     * 选择数据库(起始函数)
     * @param string $name 表名不带前缀
     */
    public static function name($table) { 
        $config = require __PATH__ .DS. 'vendor' .DS. 'config' .DS. 'database' .EXT;
        self::$_CONFIG = $config;

        # 初始化数据
        self::$_WHERE = '';
        self::$_FIELD = '*';
        self::$_ORDER = '';
        self::$_LIMIT = '';
        self::$_SQL   = '';
        self::$_DATA  = [];
        self::$_JOIN  = '';
        self::$_PREPARE = [];
        self::$_STATUS  = false;
        self::$_TABLE = self::$_CONFIG['prefix'] . $table;

        return new self;
    }

    /**
     * 表别名
     * @param string $as 别名
     */
    public static function alias($as) {
        self::$_TABLE = self::$_TABLE.' AS '.$as;
        return new self;
    }

    /**
     * JOIN链表
     * @param string $join 链表式 __USER__ B
     * @param string $where 条件  A.id = B.id
     * @param string $type 关联方式 LETF JOIN
     */
    public static function join($join, $where, $type='LEFT JOIN') {
        //$config = require_once __PATH__ .DS. 'vendor' .DS. 'config' .DS. 'database' .EXT;
        $sql  = '';
        $data = explode(' ', $join);
        $join = $config['prefix'] . strtolower(str_replace('__', '', $data[0])). ' AS '.$data[1]. ' ON';
        self::$_JOIN .= $type.' '.$join.' '.$where.' ';
        return new self;
    }

    /**
     * 条件(函数链)
     * @param array $array 二维数组，条件 [ '字段名' => ['=', '12'] ]
     */
    public static function where($array) {
        if (is_array($array)) {
            $where = '';
            foreach ($array as $key=>$val) {
                $int    = self::add_prepare($val[1]);
                $where .= $key.' '. $val[0]." {$int} AND ";
            }
            self::$_WHERE .= $where;
        } else {
            self::$_WHERE .= $array;
        }
        return new self;
    }

    /**
     * 注入数据内容(函数链)
     * @param array $data 一维数组，内容 [ '字段名' => '内容' ]
     */
    public static function data($data) {
        self::$_DATA = array_merge(self::$_DATA, $data);
        return new self;
    }

    /**
     * 查询字段(函数链)
     * @param string $field
     */
    public static function field($field) {
        self::$_FIELD = $field;
        return new self;
    }

    /**
     * 排序(函数链)
     * @param string $order
     */
    public static function order($order) {
        self::$_ORDER = $order;
        return new self;
    }

    /**
     * 分页(函数链)
     * @param int $left
     * @param int $right
     */
    public static function limit($left, $right) {
        self::$_LIMIT = "{$left},{$right}";
        return new self;
    }

    /**
     * 修改数据(终结函数)
     * @param bool $debug 调试模式开启则返回解析后的SQL语句
     */
    public static function update($debug=false) {
        self::$_SQL = 'UPDATE '. self::$_TABLE .' SET ';
        # 修改内容
        foreach (self::$_DATA as $key=>$val) {
            $int = self::add_prepare($val);
            self::$_SQL .= $key." = {$int}, ";
        }
        self::$_SQL = rtrim(self::$_SQL, ', '). ' ';
        # 有条件
        if (!empty(self::$_WHERE)) {
            self::$_SQL .=  'WHERE '. rtrim(self::$_WHERE, 'AND ').';';
        }
        # 开启调试 并且关闭预注入处理
        if ($debug && self::$_STATUS == false) {
            return self::$_SQL;
        }
        # 执行SQL语句
        $statement = self::$_PDO->prepare(self::$_SQL);
        return $statement->execute(self::$_PREPARE);
    }

    /**
     * 新增数据(终结函数)
     * @param bool $debug 调试模式开启则返回解析后的SQL语句
     */
    public static function insert($debug=false) {
        self::$_SQL = 'INSERT INTO '. self::$_TABLE .' (';
        # 左边字段名
        foreach (self::$_DATA as $key=>$val) {
            self::$_SQL .= $key.', ';
        }
        self::$_SQL = rtrim(self::$_SQL, ', '). ') VALUES(';
        # 右边字段值
        foreach (self::$_DATA as $key=>$val) {
            $int = self::add_prepare($val);
            self::$_SQL .= "{$int},";
        }
        self::$_SQL = rtrim(self::$_SQL, ', '). ');';
        # 开启调试 并且关闭预注入处理
        if ($debug && self::$_STATUS == false) {
            return self::$_SQL;
        }
        # 执行SQL语句
        $statement = self::$_PDO->prepare(self::$_SQL);
        $statement->execute(self::$_PREPARE);
        return self::$_PDO->lastInsertId();
    }

    /**
     * 删除数据(终结函数)
     * @param bool $debug 调试模式开启则返回解析后的SQL语句
     */
    public static function delete($debug=false) {
        self::$_SQL = 'DELETE FROM '. self::$_TABLE .' ';
        # 有条件
        if (!empty(self::$_WHERE)) {
            self::$_SQL .=  'WHERE '. rtrim(self::$_WHERE, 'AND ').';';
        }
        # 开启调试 并且关闭预注入处理
        if ($debug && self::$_STATUS == false) {
            return self::$_SQL;
        }
        # 执行SQL语句
        $statement = self::$_PDO->prepare(self::$_SQL);
        return $statement->execute(self::$_PREPARE);
    }

    /**
     * 查询多条数据(终结函数)
     * @param bool $debug 调试模式开启则返回解析后的SQL语句
     */
    public static function select($debug=false) {
        self::$_SQL = 'SELECT '.self::$_FIELD.' FROM '.self::$_TABLE .' '.self::$_JOIN;
        # 有条件
        if (!empty(self::$_WHERE)) {
            self::$_SQL .=  'WHERE '. rtrim(self::$_WHERE, 'AND ').' ';
        }
        # 有排序
        if (!empty(self::$_ORDER)) {
            self::$_SQL .=  'ORDER BY '. self::$_ORDER.' ';
        }
        # 有分页
        if (!empty(self::$_LIMIT)) {
            self::$_SQL .=  'LIMIT '. self::$_LIMIT.' ';
        }
        self::$_SQL = rtrim(self::$_SQL, ' ').';';
        # 开启调试 并且关闭预注入处理
        if ($debug && self::$_STATUS == false) {
            return self::$_SQL;
        } else if ($debug) {
            return new self;
        }
        # 执行SQL语句
        $statement = self::$_PDO->prepare(self::$_SQL);
        $statement->execute(self::$_PREPARE);
        # 列名索引方式
        return $statement->fetchAll(\PDO::FETCH_ASSOC);
    }

    /**
     * 查询一条数据(终结函数)
     * @param bool $debug 调试模式开启则返回解析后的SQL语句
     */
    public static function find($debug=false) {
        self::$_SQL = 'SELECT '.self::$_FIELD.' FROM '.self::$_TABLE .' '.self::$_JOIN;
        # 有条件
        if (!empty(self::$_WHERE)) {
            self::$_SQL .=  'WHERE '. rtrim(self::$_WHERE, 'AND ').' ';
        }
        # 有排序
        if (!empty(self::$_ORDER)) {
            self::$_SQL .=  'ORDER BY '. self::$_ORDER.' ';
        }
        # 有分页
        if (!empty(self::$_LIMIT)) {
            self::$_SQL .=  'LIMIT '. self::$_LIMIT.' ';
        }
        self::$_SQL = rtrim(self::$_SQL, ' ').';';
        # 是否开启调试
        if ($debug) {
            return self::$_SQL;
        }
        # 执行SQL语句
        $statement = self::$_PDO->prepare(self::$_SQL);
        $statement->execute(self::$_PREPARE);
        # 列名索引方式
        return $statement->fetch(\PDO::FETCH_ASSOC);
    }

    /**
     * 获得SQL执行后的count总数
    */
    public static function count() {
        # 执行SQL语句
        $statement = self::$_PDO->prepare(self::$_SQL);
        $statement->execute(self::$_PREPARE);
        return $statement->rowCount();
    }

    /**
     * 开给分页类专用的终结函数
     * @param string $sql 最好的页数
     */
    public static function page($sql) {
        $sql = rtrim(self::$_SQL, ';').$sql;
        # 执行SQL语句
        $statement = self::$_PDO->prepare($sql);
        $statement->execute(self::$_PREPARE);
        # 列名索引方式
        return $statement->fetchAll(\PDO::FETCH_ASSOC);
    }


    /**
     * 执行原生查询语句(终结函数)
     * @param string $sql 需要执行的sql语句
     */
    public static function query($sql) {
        # 执行SQL语句
        return self::$_PDO->query($sql);
    }

    /**
     * 执行原生查询语句外的其他SQL语句(终结函数)
     * @param string $sql 需要执行的sql语句
     */
    public static function exec($sql) {
        # 执行SQL语句
        return self::$_PDO->exec($sql);
    }

    /**
     * 进行预注入参数存储处理
     * @param string $data 存储的内容
     * @return int 存储的对应编码
     */
    private static function add_prepare($data) {
        # 只有开启预注入，才会有用
        if (self::$_STATUS == false) {
            return "'$data'";
        }
        $array = self::$_PREPARE;
        $key   = count($array)+1;
        $array[':'.$key] = $data;
        self::$_PREPARE  = $array;
        return ':'.$key;
    }
}