<?php
// +----------------------------------------------------------------------
// | 公共函数库
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

/**
 * 页面处理错误跳转
 * @param string $msg  错误提示内容
 * @param string $url  跳转链接地址，如何该参数等于false，则不进行自动跳转
 * @param int    $time 自动跳转时间(秒)
 */
function error($msg, $url='', $time=3){
    require_once __PATH__ .DS. 'vendor' .DS. 'template' .DS. 'error' .EXT;
    exit;
}

/**
 * 页面处理警告跳转
 * @param string $msg  错误提示内容
 * @param string $url  跳转链接地址
 * @param int    $time 自动跳转时间(秒)
 */
function notice($msg, $url='', $time=3){
    require_once __PATH__ .DS. 'vendor' .DS. 'template' .DS. 'notice' .EXT;
    exit;
}

/**
 * 页面处理正确跳转
 * @param string $msg  错误提示内容
 * @param string $url  跳转链接地址
 * @param int    $time 自动跳转时间(秒)
 */
function exactness($msg, $url='', $time=3){
    require_once __PATH__ .DS. 'vendor' .DS. 'template' .DS. 'exactness' .EXT;
    exit;
}

/**
 * 美化var_dump()函数，方便日常调试
 * @param mixed $var 打印内容
 */
function dump($var) {
    if ($var === true || $var === false) {
        var_dump($var);
    } else {
        $content = "<pre>\n";  
        $content .= htmlspecialchars(print_r($var, true));  
        $content .= "\n</pre>\n";  
        echo $content;  
    }
    exit;
}  

/**
 * 创建和读取SESSION
 * @param string $key 键名
 * @param string $val 键值，当为空时则是读取session对应的值
 * @return string OR bool 当返回false时，则表示读取失败
 */
function SESSION($key, $val=false) {
    # 我们先判断session是否开启，如果没有则先开启
    isset($_SESSION) || session_start();
    # 再判断是读取session，还是创建session
    if ($val != false || $val === null) {
        $_SESSION[$key] = $val;
    } else {
        if (!isset($_SESSION[$key])) {
            return false;
        }
        return $_SESSION[$key];
    }
}

/**
 * 生成随机验证码
 * @param int $num    验证码长度
 * @param int $widht  宽度
 * @param int $height 高度
 */
function Verify($num=4, $width=200, $height=60){	
	$codeSet = '2346789ABCDEFGHJKLMNPQRTUVWXY';
    $bg = imagecreatetruecolor($width, $height); 
    $color = imagecolorallocate($bg, 243, 251, 254);	

    $code = array();
    $codeNX = 0;

    for ($i = 0; $i<$num; $i++) {
        $code[$i] = strtolower($codeSet[mt_rand(0, 28)]);
        $codeNX += mt_rand(20*1.2, 20*1.8);
        imagettftext($bg, 20, mt_rand(-20, 50), $codeNX, 20*1.5, $color, __PATH__.DS.'vendor'.DS.'1.ttf', $code[$i]);
    }

    # 把校验码保存到session
    SESSION('verify', join('', $code));

    header("content-type: image/png");
    imagepng($bg); 
    imagedestroy($bg);
}

/**
 * 对比验证码
 * @param string $vif 验证码
 */
function Verify_OK($vif) {
    $verify = SESSION('verify');
    SESSION('verify', null);
    if ($vif != $verify) { 
        error('验证码错误'); 
    }
}

/**
 * SQL注入过滤
 * @param string $string : 需要过滤的字符串
 * @return string 返回过滤之后的字符串
*/
function JunSql($string) {
	$source   = 'or|and|xor|not|select|insert|update|delete|=>|<=|!=|=|%|\'|\/\*|\*|\.\.\/|\.\/|union|into|load_file|outfile';
	$badword  = explode('|',$source);
	$badword1 = array_combine($badword,array_fill(0,count($badword),'/'));
    $string   = strtr($string, $badword1);
	$string   = addslashes($string);
    return $string;
}

/*
* I方法，接受并且做SQL过滤GET,POST提交的数据
* $parameter : POST.表单的name名，例如POST.title
* $error     : 非法参数之后的返回值，默认为false
* 过滤成功之后，返回接受的数据，否则返回$error值
*/
function I($parameter, $error=false){
	$num = explode('.', $parameter);
	$PostName = strtolower($num[0]);
	$PostValue = $num[1];
	switch ($PostName) {
        case 'post':
            if (empty($_POST[$PostValue])) {
                return $error;
            }

            $value = $_POST[$PostValue];
            $valueS = JunSql($value);
            if ($valueS !==  $value) {
                $res = $error;
            }else{
                $res = $valueS;
            }

            return $res;
            break;  
        case 'get':
            if (empty($_GET[$PostValue])) {
                return $error;
            }

            $value = $_GET[$PostValue];
            $valueS = JunSql($value);
            if ($valueS !==  $value) {
                $res = $error;
            }else{
                $res = $valueS;
            }

            return $res;
            break;
        default:
            return ' 并无'.$PostName.'表单类型！';
	}
}

/**
 * 生成URL
 * @todo 无
 * @author 小黄牛
 * @version v1.1.8 + 2019.04.01
 * @deprecated 暂不弃用
 * @global 无
 * @param string $c 路由c
 * @param string $v 路由v
 * @param string $param 多余get参数
 * @return string
*/
function U($c, $v='index', $param='') {
    $url  = !empty($_SERVER['PHP_SELF']) ? $_SERVER['PHP_SELF'] : $_SERVER['URL'];
    $url .= "?c={$c}&v={$v}"; 
    if (!empty($param)) {
        $url .= '&'.ltrim($param, '&');
    }
    return $url;
}

/**
 * 输出JSON
 * @todo 无
 * @author 小黄牛
 * @version v1.1.8 + 2019.04.01
 * @deprecated 暂不弃用
 * @global 无
 * @param string $code 状态码
 * @param string $msg 说明
 * @param mixed $data 数据集
 * @return json
*/
function json($code='01', $msg='', $data=[]) {
    die(json_encode([
        'code' => "{$code}",
        'msg'  => $msg,
        'data' => $data,
    ], JSON_UNESCAPED_UNICODE));
}